/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"
/* HelloWord_db.h was generated from the DDL file HelloWord_db.ddl.
 * See subsequent examples for DDL modification.
 */
#include "HelloWorld_db.h"

/* EXAMPLE - Hello World for C */
/* Each time you run this example, a new record is created in the
 * database, then all existing records are read from the database
 * and printed.
 *
 * The DDL (Database Definition Language) is in HelloWord_db.ddl. The database
 * has been pre-built and initialized.
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open/d_close
 *      d_fillnew
 *      d_recfrst/d_recnext
 *      d_recread
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database is stored in a directory named 'HelloWord_db' in this
 * project directory.
 */


/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK            *task;
    int32_t             rdm_status;
    struct HelloRecord  Hello_input;
    struct HelloRecord  Hello_output;

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access */
    rdm_status = d_open ("HelloWorld_db", "x", task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s - %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_closetask (task);
        return (1);
    }

    /* create a record in the new database */
    strncpy (Hello_input.message, "Hello World", SIZEOF_HELLORECORD_MESSAGE);
    d_fillnew (HELLORECORD, &Hello_input, task, CURR_DB);

    /* scan and read all created records */
    /* one additional record will be created each time you run this example */
    for (rdm_status = d_recfrst (HELLORECORD, task, CURR_DB);
         rdm_status == S_OKAY;
         rdm_status = d_recnext (task, CURR_DB))
    {
        d_recread (&Hello_output, task, CURR_DB);
        printf ("%s\n", Hello_output.message);
    }

    /* close the database */
    d_close (task);
    d_closetask (task);
    printf ("Press <Enter> to continue . . .\n");
    getchar ();
}
